"""A collection of Ludic components.

This module is meant as a collection of components that could be useful
for building Ludic applications. Any contributor is welcome to add new ones.

It also serves as showcase of possible implementations.
"""

# these need to be imported as they load important styling options
from . import pages, layouts  # noqa
